unit nduWirelessGraph;
// Copyright (c) 2007-2010 MetaGeek, LLC
// Copyright (c) 2015 Maciej Kaczkowski - keit.co (port to FPC)
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// source:
// https://github.com/metageek-llc/IoctlNdis/blob/master/IoctlNdis.cs
// https://github.com/metageek-llc/inSSIDer-2/blob/master/MetaScanner/UI/Controls/ChannelView.cs
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics, Controls,
  dateutils,
  nduWireless;

type
  { TWirelessGraph }

  TBandType = (Band2400MHz = 2400, Band5000MHz = 5000);

  TWirelessGraph = class(TCustomControl)
  private
    FBandType: TBandType;
    FDriver: TWireless;
    FGraphAxisLabelColor: TColor;
    FGraphAxisLabelDimColor: TColor;
    FGraphBackColor: TColor;
    FGraphForeColor: TColor;
    FGraphHorizontalDottedLineColor: TColor;
    FGraphLegendBackColor: TColor;
    FGraphOutlineColor: TColor;
    FGraphTickColor: TColor;
    FMinFrequency,
    FMaxFrequency: Integer;
    FpixelsPerMHz: Double;
    FMinAmplitude: Integer;
    FMaxAmplitude: Integer;
    FTableAlternateRowColor: TColor;
    FTableForeColor: TColor;
    FTableGridColor: TColor;
    procedure SetBandType(AValue: TBandType);
    procedure SetGraphAxisLabelColor(AValue: TColor);
    procedure SetGraphAxisLabelDimColor(AValue: TColor);
    procedure SetGraphBackColor(AValue: TColor);
    procedure SetGraphForeColor(AValue: TColor);
    procedure SetGraphHorizontalDottedLineColor(AValue: TColor);
    procedure SetGraphLegendBackColor(AValue: TColor);
    procedure SetGraphOutlineColor(AValue: TColor);
    procedure SetGraphTickColor(AValue: TColor);
    procedure SetTableAlternateRowColor(AValue: TColor);
    procedure SetTableForeColor(AValue: TColor);
    procedure SetTableGridColor(AValue: TColor);
    function viewableRangeFreq: Integer;
    function viewableRangeAmp: Integer;
    function GraphWidth: Integer;
    function GraphHeight: Integer;
    function PixelsPerMHz: Double;
    function PixelsPerDbm: Double;
    function DbToY(db: Integer): Double;
    function amplitudeLabelSpacing: Integer;
  protected
    FMargin: TRect;
    FBitmap: TBitmap;
    procedure TextOut(ACanvas: TCanvas; X, Y: Integer; AText: string; RightJust: Boolean = False);
    procedure DrawLine(ACanvas: TCanvas; X1, Y1, X2, Y2: Integer);
    procedure DrawRect(ACanvas: TCanvas; X1, Y1, X2, Y2: Integer);
    procedure FillRect(ACanvas: TCanvas; X1, Y1, X2, Y2: Integer);
    procedure DrawBackground(ACanvas: TCanvas);
    procedure DrawGrid(ACanvas: TCanvas);
    procedure DrawLabels(ACanvas: TCanvas);
    procedure DrawNetworks(ACanvas: TCanvas);
    procedure DoOnResize; override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property BandType: TBandType read FBandType write SetBandType;
    property Margin: TRect read FMargin write FMargin;
    property Driver: TWireless read FDriver write FDriver;
    // colors
    property GraphBackColor: TColor read FGraphBackColor write SetGraphBackColor;
    property GraphOutlineColor: TColor read FGraphOutlineColor write SetGraphOutlineColor;
    property GraphTickColor: TColor read FGraphTickColor write SetGraphTickColor;
    property GraphAxisLabelColor: TColor read FGraphAxisLabelColor write SetGraphAxisLabelColor;
    property GraphAxisLabelDimColor: TColor read FGraphAxisLabelDimColor write SetGraphAxisLabelDimColor;
    property GraphForeColor: TColor read FGraphForeColor write SetGraphForeColor;
    property TableAlternateRowColor: TColor read FTableAlternateRowColor write SetTableAlternateRowColor;
    property TableGridColor: TColor read FTableGridColor write SetTableGridColor;
    property TableForeColor: TColor read FTableForeColor write SetTableForeColor;
    property GraphHorizontalDottedLineColor: TColor read FGraphHorizontalDottedLineColor write SetGraphHorizontalDottedLineColor;
    property GraphLegendBackColor: TColor read FGraphLegendBackColor write SetGraphLegendBackColor;
  end;

implementation

{ TWirelessGraph }

procedure TWirelessGraph.SetBandType(AValue: TBandType);
begin
  if FBandType=AValue then
    Exit;
  FBandType := AValue;
  case FBandType of
    Band2400MHz:
      begin
        FMinFrequency := 2400;
        FMaxFrequency := 2495;
      end;
    Band5000MHz:
      begin
        FMinFrequency := 5150;
        FMaxFrequency := 5850;
      end;
  end;
  Repaint;
end;

procedure TWirelessGraph.SetGraphAxisLabelColor(AValue: TColor);
begin
  if FGraphAxisLabelColor=AValue then Exit;
  FGraphAxisLabelColor:=AValue;
end;

procedure TWirelessGraph.SetGraphAxisLabelDimColor(AValue: TColor);
begin
  if FGraphAxisLabelDimColor=AValue then Exit;
  FGraphAxisLabelDimColor:=AValue;
end;

procedure TWirelessGraph.SetGraphBackColor(AValue: TColor);
begin
  if FGraphBackColor=AValue then Exit;
  FGraphBackColor:=AValue;
end;

procedure TWirelessGraph.SetGraphForeColor(AValue: TColor);
begin
  if FGraphForeColor=AValue then Exit;
  FGraphForeColor:=AValue;
end;

procedure TWirelessGraph.SetGraphHorizontalDottedLineColor(AValue: TColor);
begin
  if FGraphHorizontalDottedLineColor=AValue then Exit;
  FGraphHorizontalDottedLineColor:=AValue;
end;

procedure TWirelessGraph.SetGraphLegendBackColor(AValue: TColor);
begin
  if FGraphLegendBackColor=AValue then Exit;
  FGraphLegendBackColor:=AValue;
end;

procedure TWirelessGraph.SetGraphOutlineColor(AValue: TColor);
begin
  if FGraphOutlineColor=AValue then Exit;
  FGraphOutlineColor:=AValue;
end;

procedure TWirelessGraph.SetGraphTickColor(AValue: TColor);
begin
  if FGraphTickColor=AValue then Exit;
  FGraphTickColor:=AValue;
end;

procedure TWirelessGraph.SetTableAlternateRowColor(AValue: TColor);
begin
  if FTableAlternateRowColor=AValue then Exit;
  FTableAlternateRowColor:=AValue;
end;

procedure TWirelessGraph.SetTableForeColor(AValue: TColor);
begin
  if FTableForeColor=AValue then Exit;
  FTableForeColor:=AValue;
end;

procedure TWirelessGraph.SetTableGridColor(AValue: TColor);
begin
  if FTableGridColor=AValue then Exit;
  FTableGridColor:=AValue;
end;

function TWirelessGraph.viewableRangeFreq: Integer;
begin
  Result := FMaxFrequency - FMinFrequency + 1;
end;

function TWirelessGraph.viewableRangeAmp: Integer;
begin
  Result := FMaxAmplitude - FMinAmplitude + 1;
end;

function TWirelessGraph.GraphWidth: Integer;
begin
  Result := (Width - FMargin.Right - FMargin.Left);
end;

function TWirelessGraph.GraphHeight: Integer;
begin
  Result := (Height - FMargin.Top - FMargin.Bottom);
end;

function TWirelessGraph.PixelsPerMHz: Double;
begin
  Result := GraphWidth / viewableRangeFreq;
end;

function TWirelessGraph.PixelsPerDbm: Double;
begin
  Result := GraphHeight / viewableRangeAmp;
end;

function TWirelessGraph.DbToY(db: Integer): Double;
begin
  Result := (FMargin.Top + GraphHeight - ((db - FMinAmplitude) * PixelsPerDbm));
end;

function TWirelessGraph.amplitudeLabelSpacing: Integer;
begin
  if (pixelsPerDbm < 1.1) then
    Result := 20
  else
  if (pixelsPerDbm < 3.3) then
    Result := 10
  else
  if (pixelsPerDbm < 6.5) then
    Result := 5
  else
  if (pixelsPerDbm >= 6.5) then
    Result := 2;
end;

procedure TWirelessGraph.TextOut(ACanvas: TCanvas; X, Y: Integer;
  AText: string; RightJust: Boolean);
var
  style: TBrushStyle;
begin
  style := ACanvas.Brush.Style;
  ACanvas.Brush.Style := bsClear;
  if RightJust then
    ACanvas.TextOut(X - ACanvas.TextWidth(AText), Y, AText)
  else
    ACanvas.TextOut(X, Y, AText);
  ACanvas.Brush.Style := style;
end;

procedure TWirelessGraph.DrawLine(ACanvas: TCanvas; X1, Y1, X2, Y2: Integer);
begin
  ACanvas.Line(X1, Y1, X2, Y2);
end;

procedure TWirelessGraph.DrawRect(ACanvas: TCanvas; X1, Y1, X2, Y2: Integer);
begin
  ACanvas.Rectangle(X1, Y1, X1 + X2, Y1 + Y2);
end;

procedure TWirelessGraph.FillRect(ACanvas: TCanvas; X1, Y1, X2, Y2: Integer);
begin
  ACanvas.FillRect(X1, Y1, X1 + X2, Y1 + Y2);
end;

procedure TWirelessGraph.DrawBackground(ACanvas: TCanvas);
begin
  ACanvas.Brush.Color := GraphBackColor;
  ACanvas.Brush.Style := bsSolid;
  ACanvas.FillRect(0, 0, Width, Height);
  ACanvas.Pen.Style := psSolid;
end;

procedure TWirelessGraph.DrawGrid(ACanvas: TCanvas);
var
  y: Double;
  maxAmpToLabel,
  labelAmplitude: Double;
  sfAmp: string;
begin
  ACanvas.Pen.Color := GraphOutlineColor;
  ACanvas.Brush.Color := GraphBackColor;

  FillRect(ACanvas, FMargin.Left - 1, FMargin.Top - 1, graphWidth + 1, graphHeight);
  DrawRect(ACanvas, FMargin.Left - 1, FMargin.Top - 1, graphWidth + 1, graphHeight + 1);

  ACanvas.Brush.Color := GraphForeColor;
  ACanvas.Font.Orientation := 900;
  ACanvas.Font.Color := GraphLegendBackColor;

  //Draw rotated line and text
  y := (GraphHeight / 2) + ACanvas.TextWidth('[dBm]') / 2 + FMargin.Top;
  ACanvas.Brush.Style := bsClear;
  ACanvas.TextOut(8, Trunc(y), '[dBm]');

  // Y axis
  maxAmpToLabel := FMaxAmplitude - (amplitudeLabelSpacing / 3);
  labelAmplitude := Round(FMinAmplitude - (FMinAmplitude mod amplitudeLabelSpacing) + amplitudeLabelSpacing);
  //var sfAmp = new StringFormat {Alignment = StringAlignment.Far};
  sfAmp := '0';

  ACanvas.Brush.Color := GraphAxisLabelColor;
  ACanvas.Font.Orientation := 0;
  while (labelAmplitude < maxAmpToLabel) do
  begin
    // amplitude label
    y := FMargin.Top + graphHeight - ((labelAmplitude - FMinAmplitude) * pixelsPerDbm);
    TextOut(ACanvas, FMargin.Left - 5, Round(y) - 7, IntToStr(Round(labelAmplitude)), True);

    // draw the horizontal graph lines
    ACanvas.Pen.Color := GraphHorizontalDottedLineColor;
    ACanvas.Pen.Style := psDot;
    DrawLine(ACanvas, FMargin.Left, Round(y), FMargin.Left + graphWidth, Round(y));
    // Tick marks next to amplitude labels
    ACanvas.Pen.Color := GraphTickColor;
    ACanvas.Pen.Style := psSolid;
    DrawLine(ACanvas, FMargin.Left - 3, Round(y), FMargin.Left+ 1, Round(y));
    labelAmplitude := labelAmplitude + amplitudeLabelSpacing;
  end;

  //Draw floor label and tick
  y := DbToY(FMinAmplitude);
  ACanvas.Pen.Color := GraphTickColor;
  ACanvas.Pen.Style := psSolid;
  //Tick
  DrawLine(ACanvas, FMargin.Left - 3, Round(y), FMargin.Left + 1, Round(y));
  ACanvas.Pen.Color := GraphAxisLabelColor;
  //label
  TextOut(ACanvas, FMargin.Left - 5, Round(y) - 7, IntToStr(FMinAmplitude), True);
end;

procedure TWirelessGraph.DrawLabels(ACanvas: TCanvas);

  function IIF(a: Boolean; x, y: Integer): Integer;
  begin
    if a then
      Result := x
    else
      Result := y;
  end;

var
  x: Double;
  freq: Integer;
  y: Integer;
  channel: Integer;
begin
  ACanvas.Brush.Color := GraphForeColor;

  // X axis labels
  y := ClientHeight - FMargin.Bottom + 5;
  if (FBandType = Band2400MHz) then
  begin
    for channel := 0 to 13 do
    begin
      freq := 2412 + (5 * channel);
      if (channel = 13) then
      begin
        freq := freq + 7;
      end;
      x := FMargin.Left + Round(PixelsPerMHz * (freq - FMinFrequency) + (PixelsPerMHz * 0.5)); // 11 for middle of channel, 1 for 2401 MHz offset
      ACanvas.Brush.Color := GraphForeColor;
      if (channel < 9) then
      begin
        TextOut(ACanvas, Round(x - 4), y, IntToStr(channel + 1));
      end
      else
      begin
        TextOut(ACanvas, Round(x - 8), y, IntToStr(channel + 1));
      end;
    end
 end
 else
 begin
   channel := 36;
   while channel <= 165 do
   begin
     freq := 5000 + (5 * channel);
     x := FMargin.Left + (pixelsPerMHz * (freq - FMinFrequency) + (pixelsPerMHz * 0.5));
     if (channel <= 64) or (channel >= 149) then
     begin
       ACanvas.Brush.Color := GraphAxisLabelColor;
       TextOut(ACanvas, Round(x) - IIF(channel > 99, 8, 4), y, IntToStr(channel));
     end
     else
     begin
       brush.Color := GraphAxisLabelDimColor;
       TextOut(ACanvas, Round(x) - IIF(channel > 99, 8, 4), y, IntToStr(channel));
     end;

     case channel of
       64: channel := 100;
       140: channel := 149;
     else
       channel := channel + 4;
     end;
   end;
 end;
end;

procedure TWirelessGraph.DrawNetworks(ACanvas: TCanvas);
const
  COLORS: array[0..9] of TColor =
    (clRed, clYellow, clAqua,
     clLime, $0000ccff, clWhite,
     clGreen, clSilver, clFuchsia,
     clPurple);
var
  ap: TNetworkItem;
  i, j: Integer;
  x, y: Integer;
  floorY: Double;
  points: array of TPoint;
  quarterY: Double;
  topleft: TPoint;
  pointWidth: Integer;
  oldColor: TColor;

  function PointF(AX, AY: Double): TPoint;
  begin
    Result.x := round(AX);
    Result.y := round(AY);
  end;

begin
  if not Assigned(FDriver) then
    Exit;

  pointWidth := ACanvas.Pen.Width;
  oldColor := ACanvas.Font.Color;
  for i := 0 to FDriver.NetworkCount - 1 do
  begin
    ap := FDriver.NetworkItem[i];
    if (FBandType = Band2400MHz) and (ap.Channel > 14) then
      Continue;
    if (FBandType = Band5000MHz) and (ap.Channel <= 14) then
      Continue;

    j := SecondsBetween(FDriver.LastNetworkScan, ap.LastDetected);
    case Abs(j) of
      00..39: // active
        begin
          ACanvas.Pen.Color := COLORS[i mod High(COLORS)];
          ACanvas.Font.Color := COLORS[i mod High(COLORS)];
        end;
      40..69: // inactive
        begin
          ACanvas.Pen.Color := clGray;
          ACanvas.Font.Color := clGray;
        end;
      else // deleted
        Continue;
    end;

    //if not ap.Graph then
    //  Continue;
    x := Round(Margin.Left + (ap.Frequency - FMinFrequency)*PixelsPerMHz);
    y := Round(DbToY(ap.Rssi));

     // Pen pen = Pens.White;
     //try
     //{
     //pen = new Pen(
     //Color.FromArgb(ap.Age * 20 < 255 ? 255 - (ap.Age * 20) : 0, ap.MyColor), 2);
     //}
     //catch (ArgumentException)
     //{
     //}
     //if (ap.Highlight)
     //{
     //pen.Width = 4;
     //}
    if ap.Privacy = 0 then
      ACanvas.Pen.Style := psDash
    else
      ACanvas.Pen.Style := psSolid;
    if i = FDriver.NetworkIndex then
    begin
      ACanvas.Pen.Width := 4;
      ACanvas.Font.Style := ACanvas.Font.Style + [fsBold];
    end
    else
      ACanvas.Pen.Width := 2;
   // int halfChannelWidthMhz;
   //if (ap.Channel > 14)
   //{
   //halfChannelWidthMhz = 10;
   //x += (int) (3*_pixelsPerMHz);
   //}
   //else if (ap.Channel <= 14 || ap.MaxRate >= 20)
   //{
   //halfChannelWidthMhz := 10;
   //}
   //else
   //{
   //halfChannelWidthMhz = 11;
   //}
    floorY := Margin.Top + graphHeight;
    quarterY := (floorY - y)/4;

    SetLength(points, 5);
    if (ap.Is40Mhz) then
    begin
      //Debug.WriteLine("40MHz 802.11n channel", "ChannelView Draw");
      //Extend for 40Mhz channel
      if (ap.SecondaryChannelLower) then
      begin
        //Debug.WriteLine("Secondary channel lower", "ChannelView Draw");
        points[0] := PointF(x - ((ap.ChannelHalfWidth*3)*pixelsPerMHz), floorY);
        points[1] := PointF(x - (((ap.ChannelHalfWidth*3) - 1)*pixelsPerMHz), floorY - quarterY);
        points[2] := PointF(x - (((ap.ChannelHalfWidth*3) - 1.5)*pixelsPerMHz), floorY - (2*quarterY));
        points[3] := PointF(x - (((ap.ChannelHalfWidth*3) - 1.5)*pixelsPerMHz), points[2].Y - 5);
        points[4] := PointF(x - (((ap.ChannelHalfWidth*3) - 1.5)*pixelsPerMHz), y);
        ACanvas.Polyline(points);
        topleft := points[4];
        points[0].X := Round(x + ap.ChannelHalfWidth*pixelsPerMHz);
        points[1].X := Round(x + (ap.ChannelHalfWidth - 1)*pixelsPerMHz);
        points[2].X := Round(x + (ap.ChannelHalfWidth - 1.5)*pixelsPerMHz);
        points[3].X := Round(x + (ap.ChannelHalfWidth - 1.5)*pixelsPerMHz);
        points[4].X := Round(x + (ap.ChannelHalfWidth - 1.5)*pixelsPerMHz);
        ACanvas.Polyline(points);
        ACanvas.Line(points[4], topleft);
        //Set the SSID location for lower channel bonded 40MHz channel
        x := Round(x - (ap.ChannelHalfWidth * PixelsPerMHz) - ACanvas.TextWidth(ap.SSID) div 2);
        //x = (x - (int)(halfChannelWidthMhz * _pixelsPerMHz)) - (int)(stringSize.Width / 2f);
      end
      else
      begin
        //Debug.WriteLine("Secondary channel higher", "ChannelView Draw");
        points[0] := PointF(x - (ap.ChannelHalfWidth*pixelsPerMHz), floorY);
        points[1] := PointF(x - ((ap.ChannelHalfWidth - 1)*pixelsPerMHz), floorY - quarterY);
        points[2] := PointF(x - ((ap.ChannelHalfWidth - 1.5)*pixelsPerMHz), floorY - (2*quarterY));
        points[3] := PointF(x - ((ap.ChannelHalfWidth - 1.5)*pixelsPerMHz), points[2].Y - 5);
        points[4] := PointF(x - ((ap.ChannelHalfWidth - 1.5)*pixelsPerMHz), y);
        ACanvas.Polyline(points);
        topleft := points[4];
        points[0].X := Round(x + (ap.ChannelHalfWidth*3)*pixelsPerMHz);
        points[1].X := Round(x + ((ap.ChannelHalfWidth*3) - 1)*pixelsPerMHz);
        points[2].X := Round(x + ((ap.ChannelHalfWidth*3) - 1.5)*pixelsPerMHz);
        points[3].X := Round(x + ((ap.ChannelHalfWidth*3) - 1.5)*pixelsPerMHz);
        points[4].X := Round(x + ((ap.ChannelHalfWidth*3) - 1.5)*pixelsPerMHz);
        ACanvas.Polyline(points);
        ACanvas.Line(points[4], topleft);
        //Set the SSID location for upper channel bonded 40MHZ channel
        x := Round(x + (ap.ChannelHalfWidth * PixelsPerMHz) - ACanvas.TextWidth(ap.SSID) div 2);
      end;
    end
    else
    begin
     //Debug.WriteLine("20MHz channel", "ChannelView Draw");
      points[0] := PointF(x - (ap.ChannelHalfWidth *PixelsPerMHz), floorY);
      points[1] := PointF(x - ((ap.ChannelHalfWidth - 1)*PixelsPerMHz), floorY - quarterY);
      points[2] := PointF(x - ((ap.ChannelHalfWidth - 1.5)*PixelsPerMHz), floorY - (2*quarterY));
      points[3] := PointF(x - ((ap.ChannelHalfWidth - 1.5)*PixelsPerMHz), points[2].Y - 5);
      points[4] := PointF(x - ((ap.ChannelHalfWidth - 1.5)*PixelsPerMHz), y);
      ACanvas.Polyline(points);
      topleft := points[4];
      points[0].X := Round(x + ap.ChannelHalfWidth*pixelsPerMHz);
      points[1].X := Round(x + (ap.ChannelHalfWidth - 1)*pixelsPerMHz);
      points[2].X := Round(x + (ap.ChannelHalfWidth - 1.5)*pixelsPerMHz);
      points[3].X := Round(x + (ap.ChannelHalfWidth - 1.5)*pixelsPerMHz);
      points[4].X := Round(x + (ap.ChannelHalfWidth - 1.5)*pixelsPerMHz);
      ACanvas.Polyline(points);
      ACanvas.Line(points[4], topleft);
      x := x - ACanvas.TextWidth(ap.Ssid) div 2;
    end;

    y := y - 15;

    TextOut(ACanvas, x, y, ap.Ssid);
    if i = FDriver.NetworkIndex then
      ACanvas.Font.Style := ACanvas.Font.Style - [fsBold];
  end;
  ACanvas.Font.Color := oldColor;
  ACanvas.Pen.Width := pointWidth;
end;

procedure TWirelessGraph.DoOnResize;
begin
  inherited DoOnResize;
  FBitmap.Width := Width;
  FBitmap.Height := Height;
  Repaint;
end;

procedure TWirelessGraph.Paint;
begin
  inherited Paint;
  DrawBackground(FBitmap.Canvas);
  DrawGrid(FBitmap.Canvas);
  DrawLabels(FBitmap.Canvas);
  DrawNetworks(FBitmap.Canvas);

  Canvas.Draw(0, 0, FBitmap);
end;

constructor TWirelessGraph.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  GraphBackColor := clBlack;
  GraphOutlineColor := RGBToColor(120, 120, 120);
  GraphTickColor := RGBToColor(120, 120, 120);
  GraphAxisLabelColor := RGBToColor(150, 150, 150);
  GraphAxisLabelDimColor := RGBToColor(150, 150, 150);
  GraphForeColor := RGBToColor(152, 152, 152);
  TableAlternateRowColor := RGBToColor(25, 25, 25);
  TableGridColor := RGBToColor(64, 64, 64);
  TableForeColor := RGBToColor(255, 255, 255);
  GraphHorizontalDottedLineColor := RGBToColor(105, 105, 105);
  GraphLegendBackColor := RGBToColor(255, 255, 255);

  FBitmap := TBitmap.Create;
  FMargin.Bottom := 20;
  FMargin.Top := 10;
  FMargin.Left := 55;
  FMargin.Right := 10;
  FMinAmplitude := -100;
  FMaxAmplitude := -10;
  Width := 320;
  Height := 200;
  SetBandType(Band2400MHz);
end;

destructor TWirelessGraph.Destroy;
begin
  FBitmap.Free;
  inherited;
end;

end.

